Pro fors2_combine2d,exposurelist,outfluxfile,outerrfile,meanflux=meanflux,$
                    badmap1=badmap1,badmap2=badmap2,simplemean=simplemean

;+-------------------------------------------------------------------------
;
; FORS2_COMBINE2D     11/2006
;
; Combine 2D spectra.  
;
; Previously geared to combine FORS2 data taken for Lya fluorescence
; project (specific output file size and flux scaling).
;
; Currently geared for lensed z~4 LBG J0332-3557.
;
; INPUTS
;       exposurelist  - String, name of two-column ASCII file
;                       containing root names of files to be combined
;                       and the QSO flux at lambda ~ 5330.
;                       Program will look for the following files
;                       for each root name listed:
;                          <root>_sub.fits  - 2D sky-subtraced array
;                          <root>_var.fits  - 2D variance array
;                          <root>_xy.fits   - 3D coordinate array
;
; KEYWORDS
;       meanflux      - Floating scalar, flux level to which to scale
;                       exposures according to individual values in 
;                       exposurelist
;       badmap1       - String, FITS filename of bad pixel map for
;                       CCD1.  Pixels with zero value in this array
;                       will be discarded.
;       badmap2       - String, FITS filename of bad pixel map for
;                       CCD2.  Pixels with zero value in this array
;                       will be discarded.       
;       simplemean    - If set, values in the combined frame will
;                       be determined using simple averaging (after
;                       rescaling to match the mean flux and sigma
;                       clipping).  Deault is to use inverse variance
;                       weighting.  A simple average may be somewhat
;                       noisier, but the error array will tend to
;                       be more accurate, particularly if the number
;                       of exposures is large; however, the error
;                       array will also be noisy.
;
; OUTPUTS
;       outfluxfile   - String, name of FITS file to which to write
;                       combined 2D array.
;       outerrfile    - String, name of FITS file to which to write
;                       2D array of formal 1-sigma errors.
;
; HISTORY
;       Written 11/15/2006 GDB
;       BADMAP1/2 keywords added 5/11/2007 GDB
;       Adapted for J0332 3/7/2012 GDB
;       MEANFLUX keyword added 3/7/2012
;       SIMPLEMEAN keyword added 11/13/2013 GDB
;--------------------------------------------------------------------------

if (n_params() lt 3) then begin
   print,'CALLING SEQUENCE: '
   print,'     fors2_combine2d,exposurelist,outfluxfile,outerrfile,'
   print,'                     meanflux=meanflux,badmap1=badmap1,'
   print,'                     badmap2=badmap2,simplemean=simplemean'
   return
endif

;;; Read in list of files to be combined
templ = {   $
          version: 1.0, $    
          datastart: 0, $   
          delimiter: ' ', $
          missingvalue: '****', $
          commentsymbol: '#', $
          fieldcount: 2, $
          fieldtypes: [7,4], $
          fieldnames: ['root','flux'], $
          fieldlocations: [0,57], $
          fieldgroups: [0,1] $
        }
data      = read_ascii(exposurelist,template=templ)
rootname  = data.root
fluxlevel = data.flux
nfiles    = n_elements(rootname)

;;; Scale flux levels to mean level (measured in 110 exposures
;;; from 2004, 2005, and 2006)
if keyword_set(meanflux) then meanfluxlevel = meanflux else $
   meanfluxlevel = 3742. ; for longslit LAE
scale = meanfluxlevel / fluxlevel

;;; Read in bad pixel maps (optional)
if (keyword_set(badmap1) or keyword_set(badmap2)) then begin
   do_masking = 1
   if keyword_set(badmap1) then bad_map1 = readfits(badmap1) $
      else bad_map1 = 1 + lonarr(2048,1024)
   if keyword_set(badmap2) then bad_map2 = readfits(badmap2) $
      else bad_map2 = 1 + lonarr(2048,1024)
endif else begin
   do_masking = 0
endelse

;;; Output arrays

out_xsz   = 2080L
out_ysz   = 200L  ;1800L for longslit LAE
comb_flux = fltarr(out_xsz,out_ysz)
comb_err  = fltarr(out_xsz,out_ysz)

;;; In order to conserve memory, proceed in chunks of 1D indices.  For
;;; each chunk, all input files will be read in and only those pixels
;;; that fall in the chunk will be kept.

pixperchunk = 4e5   ; Max number of output pixels in each chunk
total_pix   = out_xsz*out_ysz
n_chunk     = ceil(total_pix / pixperchunk)

for i=0,n_chunk-1 do begin

   index_min = i*pixperchunk
   index_max = (i+1)*pixperchunk < (total_pix-1)
   chunk_sz  = index_max - index_min + 1

   ; Create large enough arrays to hold information from all input
   ; arrays, although not all input arrays will contribute to the
   ; current chunk.  (This is faster than concatenating contributing
   ; pixels aftr each frame.)
   temp_n     = chunk_sz * nfiles
   index_list = lonarr(temp_n)   ; Nearest neighbor sampling (integer indices) 
   flux_list  = fltarr(temp_n)
   var_list   = replicate(-1.,temp_n)   ; Default is no data
   mask_list  = replicate(1,temp_n)     ; Default is no masking
   count      = 0                       ; Index counter

   ; Read in data for this chunk
   for j=0,nfiles-1 do begin
      temp_xy    = readfits(rootname(j)+'_xy.fits')
      temp_index = temp_xy(*,*,0) + out_xsz*temp_xy(*,*,1)
      contrib_ls = where(temp_index ge index_min and $
                         temp_index le index_max,n_contrib)
      ; Only read in the remaining files for this exposure if the xy
      ; array indicates that this exposure contributes to the current
      ; chunk.
      if (n_contrib ne 0) then begin
         temp_flux  = readfits(rootname(j)+'_sub.fits',hflux)
         temp_var   = readfits(rootname(j)+'_var.fits',hvar)
         index_list(count:count+n_contrib-1) = temp_index(contrib_ls)
         flux_list(count:count+n_contrib-1)  = scale(j)*temp_flux(contrib_ls)
         var_list(count:count+n_contrib-1)   = (scale(j)^2)*temp_var(contrib_ls)
         ; Masking bad pixels?
         if do_masking then begin
            ; Determine which CCD
            chip = strtrim(sxpar(hflux,'EXTNAME'),2)
            case chip of
               'CHIP1': mask_list(count:count+n_contrib-1) = $
                                                  bad_map1(contrib_ls)
               'CHIP2': mask_list(count:count+n_contrib-1) = $
                                                  bad_map2(contrib_ls)
               else:    begin
                           print,'Cannot determine which CCD for file '+$
                                 rootname(j)
                           return
                        end
            endcase
         endif
         ; Increment pixel count
         count = count + n_contrib
      endif
   endfor

   ; Truncate lists to those pixels with good data.  Minimum
   ; value for flux was determined empirically from the data.
   good_ls    = where(var_list gt 0 and flux_list gt -75 and mask_list eq 1)
   index_list = index_list(good_ls)
   flux_list  = flux_list(good_ls)
   var_list   = var_list(good_ls)

   ; Number of sigma away from median for rejection
   if keyword_set(simplemean) then nsig = 3. else nsig = 5.

   ; Compute the combined flux in each pixel
   h = histogram(index_list,omin=ind_min,rev=ri)
   for k=0L,n_elements(h)-1 do begin     
      index = ind_min + k
      if (ri(k) eq ri(k+1)) then continue
      n_pix    = h(k)
      pix_ls   = ri(ri(k):ri(k+1)-1)
      pix_flux = flux_list(pix_ls)
      pix_var  = var_list(pix_ls)
      med      = median(pix_flux,/even)
      if (n_pix gt 2) then begin
         if (n_pix ge 4) then sdev = robust_sigma(pix_flux) $
            else sdev = stddev(pix_flux)
         keep_ls = where(abs(pix_flux-med) lt nsig*sdev,n_keep)
      endif else begin
         keep_ls = lindgen(n_pix)
         n_keep  = n_pix
      endelse
      if (n_keep ne 0) then begin
         keep_flux   = pix_flux(keep_ls)
         keep_var    = pix_var(keep_ls)
         keep_invvar = 1./keep_var
         if keyword_set(simplemean) then begin
            comb_flux(index) = mean(keep_flux)
            comb_err(index)  = stddev(keep_flux) / sqrt(n_keep)
         endif else begin
            comb_flux(index) = total(keep_flux*keep_invvar)/$
                               total(keep_invvar)
            comb_err(index)  = sqrt(1./total(keep_invvar))
         endelse
      endif else begin
         comb_flux(index) = 0.
         comb_err(index)  = -1.
      endelse
   endfor   

   ;; Go line-by-line, pixel-by-pixel and compute the combined flux
   ;for k=y_min,y_max do begin
   ;   line_ls   = where(y_list eq k,n_line)
   ;   if (n_line ne 0) then begin
   ;      line_x    = x_list(line_ls)
   ;      line_flux = flux_list(line_ls)
   ;      line_var  = var_list(line_ls)
   ;      for l=0,out_xsz-1 do begin
   ;         pix_ls = where(line_x eq l,n_pix)
   ;         if (n_pix ne 0) then begin
   ;            pix_flux = line_flux(pix_ls)
   ;            pix_var  = line_var(pix_ls)
   ;            med      = median(pix_flux,/even)
   ;            if (n_pix gt 2) then begin
   ;               if (n_pix ge 4) then sdev = robust_sigma(pix_flux) $
   ;                  else sdev = stddev(pix_flux)
   ;               keep_ls = where(abs(pix_flux-med) lt nsig*sdev,n_keep)
   ;            endif else begin
   ;               keep_ls = lindgen(n_pix)
   ;               n_keep  = n_pix
   ;            endelse
   ;            if (n_keep ne 0) then begin
   ;               keep_flux   = pix_flux(keep_ls)
   ;               keep_var    = pix_var(keep_ls)
   ;               keep_invvar = 1./keep_var
   ;               comb_flux(l,k) = total(keep_flux*keep_invvar)/$
   ;                                total(keep_invvar)
   ;               comb_err(l,k)  = sqrt(1./total(keep_invvar))
   ;            endif else begin
   ;               comb_flux(l,k) = 0.
   ;               comb_err(l,k)  = -1.
   ;            endelse
   ;         endif
   ;      endfor
   ;   endif
   ;endfor   

endfor

; Write output files

writefits,outfluxfile,comb_flux
writefits,outerrfile,comb_err


return
end
