Pro fors2_flat_combine,calibdir,b1=b1,b2=b2

;+--------------------------------------------------------------------------
;
; FORS2_FLAT_COMBINE    10/2004
;
; ****************************************
; ***** Replaced by FORS2_FLATFIXER ******
; ****************************************
;
; Combine ESO/FORS2 flat fields.  Individual frames have their pre/over-
; scan regions removed, are bias-subtracted, and scaled so that all frames
; have equal means.  It is assumed that the frames have equal exposure times,
; and so the scaling will be small.  Sclaed flats are median-combined.
;
; Currently geared to combine flats with 2" slit in spectrum mode.
; Variance is computed assuming a gain of 0.7 e-/ADU, and a read noise
; of 2.9 e-.
;
; Output flats are normalized by a cubic spline fit in the x (dispersion) 
; direction, with moderately fine spacing of spline points.
;
; Will either combine flats listed in the 'list_of_all_calibs.txt'
; file in the calibration directory included with the standard ESO
; data distribution (all matching flats in that file are combined), or
; those specified with the 'flatlist' keyword'.
; 
; Warning: If no flat field exposures are present for chip 1, the program
; will exit without looking for exposures for chip 2.
; 
; INPUTS
;         calibdir    - String, directory containing
;                       calibration files.
;
; KEYWORDS
;         b1          - Filename of bias exposure for chip 1.  Default
;                       is to use the master bias contained in the
;                       calibration directory.
;         b2          - Filename of bias exposure for chip 2.
;
; OUTPUTS
;         Combined flats and variances written to current directory.
;
; HISTORY
;         Written 10/20/2004 GDB
;----------------------------------------------------------------------------

print,'******************************************************************'
print,'**** FORS2_FLAT_COMBINE has been replaced by FORS2_FLATFIXER. ****'
print,'******************************************************************'
return

if (n_params() eq 0) then begin
   print,'CALLING SEQUENCE:'
   print,'     fors2_flat_combine,calibdir,b1=b1,b2=b2'
   return
endif

;;;
;;; Go to calibration directory
;;; 

cd,calibdir,current=currentdir

;;;
;;; Get list of flat files to combine from list from 'list_of_all_calibs.txt'
;;;

; Create a temporary file from containing only raw calibration files
; from 'list_of_all_calibs.txt'
checkexists = findfile('temp_calibs.txt',count=nfiles)
if (nfiles ne 0) then begin
   spawn,'"rm" temp_calibs.txt'
endif
spawn,'egrep " CALIB " list_of_all_calibs.txt > temp_calibs.txt'

; Template for reading 'list_of_all_calibs.txt', with lines at start of file
; removed (23 fields).

templ = {         $
         version: 1.0, $    
         datastart: 0L, $
         delimiter: ' ', $     ; Fields separated by white space
         missingvalue: -99.0, $
         commentsymbol: '#', $
         fieldcount: 23L, $
         fieldtypes: [7,7,7,4,7,7,7,7,7,7,7,7,7,3,7,3,3,4,3,7,7,7,7], $
         fieldnames: ['TARGET','RA','DEC','EXP','AIRM','FILTER','SLT',$
                      'GRIS','CL','LST','DCATG','DTYPE','DTECH','OB_ID',$
                      'TPL_ID','NX','X','CON','CHIP','BIN','WINDOWS',$
                      'archive_filename','SETUP_CHECKSUM'], $
         fieldlocations: [0,14,24,35,41,47,55,62,69,72,85,94,104,113,123,$
                          140,143,146,151,156,160,170,201], $
         fieldgroups: [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,$
                       20,21,22] $
        }

calibdata = read_ascii('temp_calibs.txt',template=templ)

; Get flatfield file names
flat1_ls    =  where(calibdata.dtype eq 'FLAT,LAMP' and $
                     calibdata.dtech eq 'SPECTRUM'  and $
                     calibdata.slt   eq '2_0'       and $
                     calibdata.chip  eq 1,nflat1)
if (nflat1 ne 0) then begin
   flat1_files = calibdata.archive_filename(flat1_ls) + '.fits'
endif else begin
   print,'FORS2_FLAT_COMBINE: ERROR - No flat files found for chip 1.'
   cd,currentdir
   return
endelse

flat2_ls    =  where(calibdata.dtype eq 'FLAT,LAMP' and $
                     calibdata.dtech eq 'SPECTRUM'  and $
                     calibdata.slt   eq '2_0'       and $
                     calibdata.chip  eq 2,nflat2)
if (nflat2 ne 0) then begin
   flat2_files = calibdata.archive_filename(flat2_ls) + '.fits'
endif else begin
   print,'FORS2_FLAT_COMBINE: ERROR - No flat files found for chip 2.'
   cd,currentdir
   return
endelse

;;;
;;; Identify and read in master bias frames
;;;

checkexists = findfile('temp_master.txt',count=nfiles)
if (nfiles ne 0) then begin
   spawn,'"rm" temp_master.txt'
endif

spawn,'egrep " MASTER_" list_of_all_calibs.txt > temp_master.txt'

mastertempl = {          $
         version: 1.0, $    
         datastart: 0L, $
         delimiter: ' ', $     ; Fields separated by white space
         missingvalue: -99.0, $
         commentsymbol: '#', $
         fieldcount: 14L, $
         fieldtypes: [7,7,7,7,7,7,4,3,7,7,4,3,7,7], $
         fieldnames: ['CALIB_NAME','FILTER','SLT','GRIS',$
                      'CL','PRO_CATG','CON','CHIP','BIN','WINDOWS',$
                      'MEAN','N','PIPE_NAME','SETUP_CHECKSUM'], $ 
         fieldlocations: [0,52,64,68,73,76,103,108,113,117,128,137,139,186], $
         fieldgroups: [0,1,2,3,4,5,6,7,8,9,10,11,12,13] $
        }

if keyword_set(b1) then begin
   bias1_name = b1
endif else begin
   masterdata = read_ascii('temp_master.txt',template=mastertempl)
   mbias1_ls = where(masterdata.pro_catg eq 'MASTER_BIAS' and $
                     masterdata.chip eq 1)
   if (total(mbias1_ls) ne -1) then begin
      n_mbias1 = n_elements(mbias1_ls)
      ; For now, use last bias created
      whichbias1 = (n_mbias1 - 1) > 0
      all_bias1_names = masterdata.calib_name(mbias1_ls)
      bias1_name = all_bias1_names(whichbias1)
   endif else begin
      print,'FORS2_FLAT_COMBINE: ERROR - No bias file found for chip 1.'
      cd,currentdir
      return
   endelse
endelse   

if keyword_set(b2) then begin
   bias2_name = b2
endif else begin
   masterdata = read_ascii('temp_master.txt',template=mastertempl)
   mbias2_ls = where(masterdata.pro_catg eq 'MASTER_BIAS' and $
                     masterdata.chip eq 2)
   if (total(mbias2_ls) ne -1) then begin
      n_mbias2 = n_elements(mbias2_ls)
      ; For now, use last bias created
      whichbias2 = (n_mbias2 - 1) > 0
      all_bias2_names = masterdata.calib_name(mbias2_ls)
      bias2_name = all_bias2_names(whichbias2)
   endif else begin
      print,'FORS2_FLAT_COMBINE: ERROR - No bias file found for chip 2.'
      cd,currentdir
      return
   endelse
endelse 

bias1 = readfits(bias1_name,hbias1)
bias2 = readfits(bias2_name,hbias2)

;;;
;;; Read flats into a large array.  Trim pre/overscan regions and
;;; bias-subtract.
;;;

bigflat1 = fltarr(2048,1024,nflat1)
bigflat2 = fltarr(2048,1024,nflat2)

bigflatmean1 = fltarr(nflat1)
bigflatmean2 = fltarr(nflat2)

for i=0,nflat1-1 do begin
   ; Read in flaat
   tmp_flat1 = readfits(flat1_files(i),hflat1)
   ; Trim pre/overscan regions
   hextract,tmp_flat1,hflat1,0,2047,5,1028
   bigflat1(*,*,i) = tmp_flat1 - bias1
   bigflatmean1(i) = mean(bigflat1(*,*,i))
endfor

for i=0,nflat2-1 do begin
   ; Read in flaat
   tmp_flat2 = readfits(flat2_files(i),hflat2)
   ; Trim pre/overscan regions
   hextract,tmp_flat2,hflat2,0,2047,5,1028
   bigflat2(*,*,i) = tmp_flat2 - bias2
   bigflatmean2(i) = mean(bigflat2(*,*,i))
endfor

;;;
;;; Scale each individual exposure by its mean, then median combine.
;;;

scale1 = bigflatmean1 / bigflatmean1(0)
scale2 = bigflatmean2 / bigflatmean2(0)

for i=0,nflat1-1 do bigflat1(*,*,i) = bigflat1(*,*,i) / scale1(i)
for i=0,nflat2-1 do bigflat2(*,*,i) = bigflat2(*,*,i) / scale2(i)

medflat1 = median(bigflat1,dim=3)
medflat2 = median(bigflat2,dim=3)

; Compute variancein the median, assuming Poisson statistics, a gain
; of 0.7 e-/ADU, and a read noise of 2.9 e-.

gn  = 0.7 ; e-/ADU
rdn = 2.9 ; e-

medvar1 = sqrt(!pi/2.) * [medflat1/gn + (rdn/gn)^2] / nflat1
medvar2 = sqrt(!pi/2.) * [medflat2/gn + (rdn/gn)^2] / nflat2

;;;
;;; Fit a cubic spline the dispersion direction to take out the spectral
;;; shape.
;;;

; Take a median slice along the x-direction.
slice1 = median(medflat1(*,350:650),dim=2)
slice2 = median(medflat2(*,350:650),dim=2)

; Spline points.
sspace   = 20.
nspline1 = round(n_elements(slice1)/sspace) 
nspline2 = round(n_elements(slice2)/sspace) 
xs1 = sspace/2. + sspace*findgen(nspline1)
xs2 = sspace/2. + sspace*findgen(nspline2)
ys1 = fltarr(nspline1)
ys2 = fltarr(nspline2)
for i=0,nspline1-1 do begin
   lo = (xs1(i) - sspace/2.) > 0
   hi = (xs1(i) + sspace/2. - 1) < (n_elements(slice1) - 1)
   ys1(i) = median(slice1(lo:hi))
endfor
for i=0,nspline2-1 do begin
   lo = (xs2(i) - sspace/2.) > 0
   hi = (xs2(i) + sspace/2. - 1) < (n_elements(slice2) - 1)
   ys2(i) = median(slice2(lo:hi))
endfor

; Calculate spline
yinit1 = spl_init(xs1,ys1)
yinit2 = spl_init(xs2,ys2)

; Interpolate new values
xinterp1 = findgen(2048)
xinterp2 = findgen(2048)
yinterp1 = spl_interp(xs1,ys1,yinit1,xinterp1)
yinterp2 = spl_interp(xs2,ys2,yinit2,xinterp2)

; Divide median flats by spline fit
normflat1 = 0.*medflat1
normvar1  = 0.*medvar1
normflat2 = 0.*medflat2
normvar2  = 0.*medvar2
for i=0,1023 do begin
   normflat1(*,i) = medflat1(*,i) / yinterp1
   normvar1(*,i)  = medvar1(*,i) / (yinterp1^2)
   normflat2(*,i) = medflat2(*,i) / yinterp2
   normvar2(*,i)  = medvar2(*,i) / (yinterp2^2)
endfor

;;;
;;; Write output files
;;;

newhdr1 = hflat1
sxaddhist,'----------------------------------------------------------',newhdr1
sxaddhist,'Flatfield  combination done using FORS2_FLAT_COMBINE (GDB)',newhdr1
sxaddhist,'   Bias frame: '+bias1_name,newhdr1
sxaddhist,'   Flatfield frames combined:',newhdr1
for i=0,nflat1-1 do $
   sxaddhist,'      '+flat1_files(i),newhdr1
sxaddhist,'----------------------------------------------------------',newhdr1

newhdr2 = hflat2
sxaddhist,'----------------------------------------------------------',newhdr2
sxaddhist,'Flatfield  combination done using FORS2_FLAT_COMBINE (GDB)',newhdr2
sxaddhist,'   Bias frame: '+bias2_name,newhdr2
sxaddhist,'   Flatfield frames combined:',newhdr2
for i=0,nflat2-1 do $
   sxaddhist,'      '+flat2_files(i),newhdr2
sxaddhist,'----------------------------------------------------------',newhdr2

writefits,'norm_spec_flat_1.fits',normflat1,newhdr1
writefits,'norm_spec_flat_1_var.fits',normvar1,newhdr1

writefits,'norm_spec_flat_2.fits',normflat2,newhdr2
writefits,'norm_spec_flat_2_var.fits',normvar2,newhdr2

;;;
;;; Return to current directory
;;;

cd,currentdir

return
end
