Pro fors2_flatfixer,flatlist,wavearr,slitarr,combflat,combflatvar,$
                    ccd=ccd,range=range,bias=bias,gain=gain,rdnoise=rdnoise,$
                    illum=illum

;+-------------------------------------------------------------------
;
; FORS2_FLATFIXER    5/2004
;
; Median-combine and normalize FORS2 longslit flats.
;
; Input flats are median combined, the combined flat is normalized,
; and the formal variance in the normalized flat is computed assuming
; Poisson statistics.  Pre/overscan regions are removed, if present.
;
; Currently assumes 2x2 binning wih ~0.6 AA/binned pixel.
;
; INPUTS
;       flatlist  - String, name of single-column ASCII file containing
;                   the FITS filenames of flats to be combined.
;       wavearr   - 2D array or fits filename, where the pixel values
;                   are wavelengths in agstroms.
;       slitarr   - 2D array or fits filename, where the pixel values
;                   are position along the slit.
;
; KEYWORDS
;       ccd       - Scalar, CCD number (1 or 2).  MUST BE SET!
;       range     - Two-element array giving max and min slit
;                   positions of the illuminated area.  Regions
;                   outside this range will be filled with 1's in the
;                   normalized flat and -1's in the variance array.  Actual
;                   fitting is done in a slightly narrower window, to avoid
;                   edge effects.  Default is [-1e6,1022] for CCD1 (goes 
;                   to bottom edge of array), and [362,1e6] for CCD2 (goes
;                   to top edge of array).  Non-illuminated rows are
;                   automatically avoided.
;       bias      - 2D array or fits filename, bias frame to be
;                   subtracted.  Default is to measure bias level
;                   from prescan region (recommended).
;       gain      - Scalar, gain in e-/ADU.  Default is 0.7.
;       rdnoise   - Scalar, read noise, in e-.  Default is 2.9.
;       illum     - Scalar, polymonial order+1 used to fit the slit
;                   illumination functin (1=constant, 2=linear, 3=
;                   quadrati, etc.).  Default is 1 (constant).
;
;  OUTPUTS
;       combflat    - Median-combined, normalized flat field.  May
;                     either be a variable or a fits filename to which
;                     array is to be written.
;       combflatvar - Formal variance in the combined, normlized flat.
;                     May either be a variable or a fits filename to
;                     which array is to be written.
;
; HISTORY
;       Written 10/30/2006 GDB
;---------------------------------------------------------------------

if (n_params() eq 0) then begin
   print,'CALLING SEQUENCE:  '
   print,' fors2_flatfixer,flatlist,wavearr,slitarr,combflat,combflatvar,'
   print,'                 ccd=ccd,range=range,gain=gain,rdnoise=rdnosie,'
   print,'                 illum=illum'
   return
endif

;;; Determine whether this is CCD1 or CCD2
if keyword_set(ccd) then begin
   if not(ccd eq 1 or ccd eq 2) then begin
      print,'FORS2_FLATFIXER: Must set ccd=1 or ccd=2.'
      return
   endif
endif else begin
   print,'FORS2_FLATFIXER: Must set ccd=1 or ccd=2.'
   return
endelse

;;; Read in flat filenames
templ = {   $
       version: 1.0, $
       datastart: 0L, $
       delimiter: ' ', $
       missingvalue: -99.0, $
       commentsymbol: '#', $
       fieldcount: 1L, $
       fieldtypes: [7L], $
       fieldnames: ['name'], $
       fieldlocations: [6L], $
       fieldgroups: [0L] $
     }
list     = read_ascii(flatlist,template=templ)
flatfile = list.name
nflat    = n_elements(flatfile)

;;; Read in wave and slit grids.

if (size(wavearr,/tname) eq 'STRING') then begin
   checkexists = findfile(wavearr,count=nfiles)
   if (nfiles eq 0) then begin
      print,'Wavelength file not found.'
      return
   endif else begin
      wavegrid = readfors2(wavearr)
   endelse
endif else wavegrid = wavearr

if (size(slitarr,/tname) eq 'STRING') then begin
   checkexists = findfile(slitarr,count=nfiles)
   if (nfiles eq 0) then begin
      print,'Slit position file not found.'
      return
   endif else begin
      slitgrid = readfors2(slitarr)
   endelse
endif else slitgrid = slitarr

;;;
;;; Median combine flats
;;;

; Arrays to store processed flats.
sz         = size(wavegrid)
xsz        = sz(1)
ysz        = sz(2)
bigflat    = fltarr(xsz,ysz,nflat)
texp       = fltarr(nflat)         ; Exposure times

; Default is measure bias from prescan region
if keyword_set(bias) then begin
   sub_bias = 0
endif else begin
   sub_bias = 1
   biasvar  = 0.
   biasname = 'Prescan Region'   
endelse 

; Read in flats
for k=0,nflat-1 do begin
   tmp_flatfile = flatfile(k)
   checkexists  = findfile(tmp_flatfile,count=nfiles)
   if (nfiles eq 0) then begin
      print,'Flat file '+tmp_flatfile+' not found.'
      return
   endif else begin
      bigflat(*,*,k) = readfors2(tmp_flatfile,flat_hdr,subbias=sub_bias)
      texp(k)        = sxpar(flat_hdr,'EXPTIME')
   endelse
endfor

; Check that all flats have similar exposure times
texp_fracdiff = abs(texp - mean(texp))/mean(texp)
if (max(texp_fracdiff) gt 0.01) then begin
   print,'FORS2_FLATFIXER: Different exposure times detected.'
   return
endif

; Median combine
medflat = median(bigflat,dim=3,/even)

;;;
;;; 2D Bias subtract (optional)
;;;

if keyword_set(bias) then begin
   if (size(bias,/tname) eq 'STRING') then begin
      checkexists = findfile(bias,count=nfiles)
      if (nfiles eq 0) then begin
         print,'Bias file not found.'
         return
      endif else begin
         biasarr  = readfors2(bias)
         biasname = bias
      endelse
   endif else begin
      biasarr  = bias
      biasname = 'Unknown'
   endelse
   medflat = medflat - biasarr
   ; Measure variance in the bias
   biassig = robust_sigma(biasarr(800:1200,300:700))
   biasvar = biassig^2
endif

;;;
;;; Normalize
;;;

; Set gain and read noise
if keyword_set(gain) then gn = gain else gn = 0.7         ; e-/ADU
if keyword_set(rdnoise) then rdn = rdnoise else rdn = 2.9 ; e-
   
; Identify illuminated region
xarr = lonarr(xsz,ysz)
yarr = lonarr(xsz,ysz)
for i=0,xsz-1 do xarr(i,*) = i
for i=0,ysz-1 do yarr(*,i) = i 
if keyword_set(range) then begin
   slitmin = range(0)
   slitmax = range(1)
endif else begin
   case ccd of
      1: begin
            slitmin = -1e6
            slitmax = 1022
         end
      2: begin 
            slitmin = 362
            slitmax = 1e6
         end
   endcase
endelse

; Avoid non-illumated rows
case ccd of
   1:  illum_ls = where(xarr le xsz-2 and yarr ge 2 and $
                        slitgrid ge slitmin and slitgrid le slitmax,n_illum)
   2:  illum_ls = where(xarr le xsz-2 and yarr le ysz-3 and $
                        slitgrid ge slitmin and slitgrid le slitmax,n_illum)
endcase

; Identify region to be fit.  This excludes the under-illuminated
; corners and gives a small buffer at the end of the slit.
case ccd of
   1:  fit_ls = where(xarr le xsz-2 and yarr ge 2 and $
                      slitgrid ge slitmin+10 and slitgrid le slitmax-10 and $
                      -1.19*xarr + yarr le 752 and $
                      1.18*xarr + yarr le 3140,n_fit)
   2:  fit_ls = where(xarr le xsz-2 and yarr le ysz-3 and $
                      slitgrid ge slitmin+10 and slitgrid le slitmax-10 and $
                      1.31*xarr + yarr ge 531 and $
                      -1.43*xarr + yarr ge -2353,n_fit)
endcase

; Polynomial order for slit illumination function
if keyword_set(illum) then illum_order = illum else illum_order = 1

; Fit a slowly-varying bspline in the dispersion direction.  At
; present, sampling is fine enough to remove medium-scale features.

fitslit = slitgrid(fit_ls)
fitwave = wavegrid(fit_ls)
fitflat = medflat(fit_ls)
; Assume Poisson noise
if (nflat gt 1) then med_factor = !pi/2. else med_factor = 1
fitflat_var = med_factor*(fitflat/gn + (rdn/gn)^2)/nflat + biasvar
fitflat_invvar = 1./fitflat_var

; Break point spacing for b-spline fit
bk_space = 10.

; Identify potential bad pixels before fitting.
print,' '
print,'Pre-filtering bad pixels...'
maxwave = max(fitwave)
minwave = min(fitwave)
first_filter = 1
filter_sz = 5*bk_space
n_filters = ceil((maxwave-minwave)/filter_sz)
; Default is not to filter any points
filter_ls = -1
; Speed things up by dividing arrays into regions that contain
; several filter bins.
filters_per_region = 1.*round(sqrt(n_filters))
num_regions        = ceil(1.*n_filters / filters_per_region)
for r=0,num_regions-1 do begin
   i_lo = r*filters_per_region
   i_hi = ((r+1)*filters_per_region - 1) < (n_filters - 1)
   region_wavelo = minwave + i_lo*filter_sz
   region_wavehi = minwave + (i_hi+1)*filter_sz
   region = where(fitwave ge region_wavelo and $
                  fitwave lt region_wavehi,n_region)
   if (n_region gt 0) then begin
      region_wave = fitwave(region)
      region_flat = fitflat(region)
      for i=i_lo,i_hi do begin
         wavelo = minwave + i*filter_sz
         wavehi = minwave + (i+1)*filter_sz
         bin_ls = where(region_wave ge wavelo and region_wave le wavehi,$
                        n_bin)
         if (n_bin gt 1) then begin
            tmp_flat   = region_flat(bin_ls) 
            tmp_median = median(tmp_flat)
            tmp_sdev   = robust_sigma(tmp_flat)
            tmp_filter_ls = where(abs(tmp_flat-tmp_median) ge 5*tmp_sdev,$
                                   n_tmp_filter)
            if (n_tmp_filter gt 0) then begin
               if first_filter then begin
                  filter_ls    = region(bin_ls(tmp_filter_ls))
                  first_filter = 0 
               endif else begin
                  filter_ls = [filter_ls,region(bin_ls(tmp_filter_ls))]
               endelse
            endif
         endif
      endfor
   endif
endfor
fitflat_invvar(filter_ls) = 0

; Perform b-spline fit
sset = bspline_iterfit(fitwave,fitflat,invvar=fitflat_invvar,$
                       bkspace=bk_space,$
                       yfit=fitflat_model,x2=fitslit,npoly=illum_order,$
                       upper=5,lower=5)

   ;; After the first pass, estimate variance in the fitted region
   ;; and mask strong outliers.
   ;if (i eq 0) then  begin
   ;   fitflat_var = med_factor*(fitflat_model/gn + (rdn/gn)^2)/nflat + biasvar
   ;   fitflat_invvar = 1./fitflat_var
   ;   bad_ls = where(abs(fitflat-fitflat_model) ge 20*sqrt(fitflat_var),n_bad)
   ;   if (n_bad ne 0) then begin
   ;      fitflat_invvar(bad_ls) = 0
   ;   endif
   ;endif

; Evaluate bspline fit over entire array and divide out
medflat_model = fltarr(xsz,ysz)
medflat_model(illum_ls) = bspline_valu(wavegrid(illum_ls),sset,$
                                       x2=slitgrid(illum_ls))
normflat = 1 + fltarr(xsz,ysz)
normflat(illum_ls) = medflat(illum_ls) / medflat_model(illum_ls)

; Estimate variance in the normalized flat.  Note: this estimate will
; be wrong in the under-illuminated corners!  Non-illuminated regions 
; are flagged with -1.
normflat_var = -1 + fltarr(xsz,ysz)
normflat_var(illum_ls) = $
      (med_factor*(medflat_model(illum_ls)/gn + (rdn/gn)^2)/nflat + biasvar) / $
      (double(medflat_model(illum_ls))^2)

;;;;
;;;; Fit and divide out features in the spatial direction (streaks)
;;;;
;;;; NOTE: This does not fix the streaks in the final reduced frames,
;;;; which are due to bad rows.  Fitting out spatial features is not
;;;; recommended, as they may be necessary for proper flatfielding.
;;;;
;
;; Define new regions to be fit.  These excludes the under-illuminated
;; corners but include the entire length of the slit.
;case ccd of
;   1:  fit_ls2 = where(xarr le xsz-2 and yarr ge 2 and $
;                       slitgrid ge slitmin and slitgrid le slitmax and $
;                       -1.19*xarr + yarr le 752 and $
;                       1.18*xarr + yarr le 3140,n_fit)
;   2:  fit_ls2 = where(xarr le xsz-2 and yarr le ysz-3 and $
;                       slitgrid ge slitmin and slitgrid le slitmax and $
;                       1.31*xarr + yarr ge 531 and $
;                       -1.43*xarr + yarr ge -2353,n_fit)
;endcase
;fitslit2    = slitgrid(fit_ls2)
;fitnormflat = normflat(fit_ls2)
;fitnormflat_invvar = 1./(normflat_var(fit_ls2))
;
;; Fit spatial profile across the slit
;sset2 = bspline_iterfit(fitslit2,fitnormflat,bkspace=1,$
;                        invvar=fitnormflat_invvar)
;
;; Divide out profile
;spatial_model = 1 + fltarr(xsz,ysz)
;spatial_model(illum_ls) = bspline_valu(slitgrid(illum_ls),sset2)
;normflat     = normflat / spatial_model
;normflat_var = normflat_var / (spatial_model^2)


;;;
;;; Set output arrays or write fits files
;;;

hnormflat = flat_hdr
sxaddhist,'-------------------------------------------------------',hnormflat
sxaddhist,'Flatfield processing done using FORS2_FLATFIXER',hnormflat
sxaddhist,'   2D bias frame: '+biasname,hnormflat
sxaddhist,'   Flatfield frames combined:',hnormflat
for i=0,nflat-1 do $
   sxaddhist,'      '+flatfile(i),hnormflat
sxaddhist,'-------------------------------------------------------',hnormflat

if (size(combflat,/tname) eq 'STRING') then begin
   print,'Writing '+combflat
   writefits,combflat,normflat,hnormflat
endif else combflat = normflat

if (size(combflatvar,/tname) eq 'STRING') then begin
   print,'Writing '+combflatvar
   writefits,combflatvar,normflat_var,hnormflat
endif else combflatvar = normflat_var

print,'Done.'

return
end
