Pro fors2_get_exposures,rawdir,script=script,help=help

;+--------------------------------------------------------------------------
;
; FORS2_GET_EXPOSURES    1/2005
;
; Read a 'list_of_all_obs.txt' file and print the names of the science
; exposures to files.
;
; INPUTS
;         rawdir      - Directory containing calibration files (optional).
;                       Default is the current directory.
;
; KEYWORDS
;         script      - Write reduction script to 'reduce.scr' in the 
;                       rawdir.  If script is a string, write reduction
;                       script to a file of that name.
;         help        - Print syntax.
;
; OUTPUTS
;         Filenames of science exposures are written to the file
;         'science_obs_1.txt' (CCD1) and 'science_obs_2.txt' (CCD2) in
;         the rawdir (or current directory).
;
; HISTORY
;         Written 1/13/2004 GDB
;----------------------------------------------------------------------------

if keyword_set(help) then begin
   print,'CALLING SEQUENCE: fors2_get_exposures,rawdir,/script,/help'
   return
endif

;;;
;;; Go to calibration directory (optional)
;;; 

if (n_params() ge 1) then begin
   cd,rawdir,current=currentdir
endif else begin
   cd,'.',current=currentdir
   rawdir = currentdir
endelse

;;;
;;; Get list of science exposures from 'list_of_all_obs.txt'.
;;;

; Create a temporary file from containing only raw observation files
; from 'list_of_all_obs.txt'
checkexists = findfile('temp_obs.txt',count=nfiles)
if (nfiles ne 0) then begin
   spawn,'"rm" temp_obs.txt'
endif
spawn,'egrep " SPECTRUM " list_of_all_obs.txt > temp_obs.txt'

; Template for reading 'list_of_all_obs.txt', with lines at start of file
; removed (23 fields).

templ = {         $
         version: 1.0, $    
         datastart: 0L, $
         delimiter: ' ', $     ; Fields separated by white space
         missingvalue: -99.0, $
         commentsymbol: '#', $
         fieldcount: 23L, $
         fieldtypes: [7,7,7,4,7,7,7,7,7,7,7,7,7,3,7,3,3,4,3,7,7,7,7], $
         fieldnames: ['TARGET','RA','DEC','EXP','AIRM','FILTER','SLT','GRIS',$
                      'CL','LST','DCATG','DTYPE','DTECH','OB_ID','TPL_ID',$
                      'NX','X','CON','CHIP','BIN','WINDOWS',$
                      'archive_filename','SETUP_CHECKSUM'], $
         fieldlocations: [0,14,24,35,41,47,55,62,69,72,85,94,104,113,123,140,$
                          143,146,151,156,160,170,201], $
         fieldgroups: [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,$
                       21,22] $
        }

calibdata = read_ascii('temp_obs.txt',template=templ)

; Get science exposure file names
ccd1_ls    =  where(calibdata.dtype  eq 'SKY'          and $
                    calibdata.dtech  eq 'SPECTRUM'     and $
                    calibdata.slt    eq '2_0'          and $
                    calibdata.target eq 'DMSB2139-040' and $
                    calibdata.chip   eq 1, n_ccd1)
if (n_ccd1 ne 0) then begin
   ccd1_root  = calibdata.archive_filename(ccd1_ls)
   ccd1_files = ccd1_root + '.fits'
   nccd1      = n_elements(ccd1_ls)
endif else begin
   print,'FORS2_GET_EXPOSURES: No science files found for chip 1.'
   cd,currentdir
   return
endelse

ccd2_ls    =  where(calibdata.dtype  eq 'SKY'          and $
                    calibdata.dtech  eq 'SPECTRUM'     and $
                    calibdata.slt    eq '2_0'          and $
                    calibdata.target eq 'DMSB2139-040' and $
                    calibdata.chip   eq 2, n_ccd2)
if (n_ccd2 ne 0) then begin
   ccd2_root  = calibdata.archive_filename(ccd2_ls)
   ccd2_files = ccd2_root + '.fits'
   nccd2      = n_elements(ccd2_ls)
endif else begin
   print,'FORS2_GET_EXPOSURES: No science files found for chip 2.'
   cd,currentdir
   return
endelse

; Write filenames to files
openw,1,'science_obs_1.txt'
for i=0,nccd1-1 do printf,1,ccd1_files(i)
close,1

openw,1,'science_obs_2.txt'
for i=0,nccd2-1 do printf,1,ccd2_files(i)
close,1

;;;
;;; Write reduction script (optional)
;;;

if keyword_set(script) then begin

   if (size(script,/tname) eq 'STRING') then script_name = script else $
      script_name = 'reduce.scr'

   openw,1,script_name

   ; Top directory for this date
   cd,rawdir+'/..'
   cd,rawdir,current=topdir

   ; Calibration files

   printf,1,"cd,'"+topdir+"/'"
   printf,1,"rawdir    = 'raw/'"
   printf,1,"caldir    = 'calib/'"
   printf,1,"reduxdir1 = '/scr2/gdb/eso/redux/ccd1/'"
   printf,1,"reduxdir2 = '/scr2/gdb/eso/redux/ccd2/'"
   printf,1,"wavegrid1 = '/scr2/gdb/eso/calibration/wavegrid_1.fits'"
   printf,1,"wavegrid2 = '/scr2/gdb/eso/calibration/wavegrid_2.fits'"
   printf,1,"slitgrid1 = '/scr2/gdb/eso/calibration/b0grid_1.fits'"
   printf,1,"slitgrid2 = '/scr2/gdb/eso/calibration/b0grid_2.fits'"
   printf,1,"model     = '/scr2/gdb/eso/calibration/sky_model.dat'"
   printf,1,"skylines  = '~/calib/fors/linelists/blue_skylines.dat'"
   printf,1,"bias1     = caldir+'F2_MBIA_040516A_C1hi_2x2.fits'"
   printf,1,"bias2     = caldir+'F2_MBIA_040516A_C2hi_2x2.fits'"
   printf,1,"flat1     = caldir+'norm_spec_flat_1.fits'"
   printf,1,"flat2     = caldir+'norm_spec_flat_2.fits'"
   printf,1,"flatvar1  = caldir+'norm_spec_flat_1_var.fits'"
   printf,1,"flatvar2  = caldir+'norm_spec_flat_2_var.fits'"

   ; CCD1

   for i=0,nccd1-1 do begin
      printf,1," "
      printf,1,"root = '"+ccd1_root(i)+"'"
      printf,1,"longslit_reduce,rawdir+root+'.fits',wavegrid1,slitgrid1,skymodel,objmodel,$"
      printf,1,"   /fors2_1,bias=bias1,flatfield=flat1,flatvar=flatvar1,texp=2999,$"
      printf,1,"   linelist=skylines,/waveshift,wavemodel=model,outfile=reduxdir1+root,$"
      printf,1,"   /maskall,/noshow,subwidth=1e6,slitpos=175"
      printf,1,"retall"
   endfor 

   ; CCD2

   for i=0,nccd2-1 do begin
      printf,1," "
      printf,1,"root = '"+ccd2_root(i)+"'"
      printf,1,"longslit_reduce,rawdir+root+'.fits',wavegrid2,slitgrid2,skymodel,objmodel,$"
      printf,1,"   /fors2_2,bias=bias2,flatfield=flat2,flatvar=flatvar2,texp=2999,$"
      printf,1,"   linelist=skylines,/waveshift,wavemodel=model,outfile=reduxdir2+root,$"
      printf,1,"   /maskall,/noshow,subwidth=1e6"
      printf,1,"retall"
   endfor 

   printf,1," "
   printf,1," "

   close,1

endif

cd,currentdir

return
end
