Pro fors2_slitgrid,flatfile,slitgrid

;+-------------------------------------------------------------------------
;
; FORS2_SLITGRID         9/2006
;
; Trace streaks in 2D flats to create a 2D array of slit positions.
; The slit position at a pixel is the y-intercept of the curve of
; constant wavelength that would pass through the center of that pixel.
;
; Recommended to trace the "streaks" in a multi-object flat where the
; ends of the slits overlap.  The resulting slitgrid can be used for a
; standard longslit exposure.
;
; INPUTS
;      flatfile     - String, name of flatfield FITS file
;
; KEYWORDS
;
; OUTPUTS
;      slitgrid     - 2D array or FITS flatfile, slit positions for
;                     each pixel.  If given as a string, the output
;                     will be written to a file with that name.
;
; HISTORY
;      Adopted from LRIS_SLITGRID 10/24/2006 GDB
;--------------------------------------------------------------------------

if (n_params() lt 2) then begin
 print,'CALLING SEQUENCE: '
 print,'   fors2_slitgrid,flatfile,slitgrid'
 return
endif

;;; Read in flat field and remove pre/overscan regions, if not already
;;; done.
flat = readfits(flatfile,hflat)
sz   = size(flat)
xsz  = sz(1)
ysz  = sz(2)
if (ysz ne 1024) then begin
   flat = flat(*,5:1028)
   sz  = size(flat)
   xsz = sz(1)
   ysz = sz(2)
endif
xcenter = round(xsz/2.)
ycenter = round(ysz/2.)

;;; Display a region of the flat and allow the user to 
;;; identify streaks
xmin = xcenter - 100.
xmax = xcenter + 100.
ymin = 15
ymax = 1000.
medval  = median(flat(xmin:xmax,ymin:ymax))
chan,9
loadct,0
doit,flat(xmin:xmax,ymin:ymax),0.8*medval,1.2*medval
print,'Click on image to select lines to trace.'
getpoints,x0_raw,y0_raw,/device,/down
x0 = x0_raw + xmin
y0 = y0_raw + ymin

;;; Trace each line and fit the traced points
trace_order = 2   ; (Cubic term was found to be insignificant.)
nlines      = n_elements(x0)
a           = fltarr(trace_order+1,nlines)
chan,11
ntiles2 = floor(sqrt(nlines))
ntiles1 = ceil((1.*nlines)/ntiles2)
!p.multi=[0,ntiles1,ntiles2]
for i=0,nlines-1 do begin
   chan,11
   traceline,flat,x0(i),y0(i),xtr,ytr,/xdir,ydir=ydir,nside=10,$
             /bygauss,median=7,merit=merit,/noplot
   pixtr = xtr
   ztr   = ytr
   ; Fit trace points.  Reject outliers and refit until fit converges.
   fit_converged = 0
   old_zfit      = 0
   n_iter        = 0
   good_ls       = where(merit ne 0)
   while not(fit_converged) do begin
      temp_a   = poly_fit(pixtr(good_ls),ztr(good_ls),trace_order,$
                          yerror=zerr)
      zfit = 0.*pixtr
      for j=0,trace_order do zfit = zfit + temp_a(j)*pixtr^j
      zfit_diff = zfit - old_zfit
      if (max(abs(zfit_diff)) eq 0) then begin
         fit_converged = 1
      endif else begin
         good_ls = where(abs(ztr-zfit) le 4*zerr and merit ne 0,n_good)
         if (n_good lt trace_order+1) then begin
            ; Flag bad fit
            print,'Bad fit - rejecting.'
            temp_a(*)     = -9999
            fit_converged = 1
         endif
      endelse
      n_iter = n_iter + 1
      if (n_iter ge 10) then fit_converged = 1
   endwhile
   a(*,i) = temp_a
   ;; Plot fit residuals
   ;zresid = ztr - zfit
   ;zresid_sdev = robust_sigma(zresid)
   ;chan,12
   ;plot,pixtr,zresid,/xstyle,/ystyle,psym=3,title='Trace residuals',$
   ;   xtitle='x',ytitle='y',xrange=[-5*zresid_sdev,5*zresid_sdev]
   ; Plot trace with fit
   plot,xtr,ytr,/xstyle,/ystyle,psym=3,xtitle='x',ytitle='y',charsize=1.5
   oplot,pixtr,zfit
endfor

;;; Fit coefficients as a function of x (or y)-intercept.
p  = fltarr(trace_order,4)
if (nlines gt 1) then begin
   ; Reject bad fits
   fitls = where(a(0,*) ne -9999 and a(0,*) ge -500 and a(0,*) le 1500)
   a_0 = a(0,fitls)
   chan,10
   !p.multi = [0,1,1]
   for i=0,trace_order-1 do begin
      print,' '
      print,'Fitting term '+strtrim(i+1,2)
      a_i = a(i+1,fitls)
      p_i = inter_poly_fit(a_0,a_i,1,include=inc)
      p(i,0:n_elements(p_i)-1) = p_i
   endfor
endif else begin
   p(*,0) = a(1:trace_order)
endelse

;;; Construct the slit position grid.
print,'Computing slit grid...'
x = indgen(xsz)
y = indgen(ysz)
slitarr = float(xy2b(x,y,p))

;;; Optional - write to file.
if (size(slitgrid,/tname) eq 'STRING') then $
   writefits,slitgrid,slitarr,hflat else slitgrid = slitarr

print,'Done.'

return
end
