Pro fors2_stacksky,exposurelist,outskyfile,outwavfile

;+-------------------------------------------------------------------------
;
; FORS2_STACKSKY     4/2008
;
; Stack model sky spectra from multiple exposures.  Currently geared
; to combine FORS2 data taken for Lya fluorescence project.
;
; INPUTS
;       exposurelist  - String, name of two-column ASCII file
;                       containing root names of files to be combined
;                       and the QSO flux at lambda ~ 5330.
;                       Program will look for the following files
;                       for each root name listed:
;                          <root>_sky.fits  - 2D mode sky array
;                          <root>_wav.fits  - 2D wavelength array
;
; KEYWORDS
;
; OUTPUTS
;       outskyfile    - String, name of FITS file to which to write
;                       combined sky spectrum.
;       outwavfile    - String, name of FITS file to which to write
;                       wavelengths for combined sky spectrum.
;
; HISTORY
;       Written 4/21/2008 GDB
;--------------------------------------------------------------------------

if (n_params() lt 1) then begin
   print,'CALLING SEQUENCE: '
   print,'     fors2_stacksky,exposurelist,outskyfile,outwavfile'
   return
endif

;;; Read in list of files to be combined
templ = {   $
          version: 1.0, $    
          datastart: 0, $   
          delimiter: ' ', $
          missingvalue: '****', $
          commentsymbol: '#', $
          fieldcount: 3, $
          fieldtypes: [7,4,4], $
          fieldnames: ['root','flux','ccdnum'], $
          fieldlocations: [0,57,65], $
          fieldgroups: [0,1,2] $
        }
data      = read_ascii(exposurelist,template=templ)
rootname  = data.root
fluxlevel = data.flux
ccdnum    = data.ccdnum
nfiles    = n_elements(rootname)

;;; Central wavelength around which to extract and number
;;; of pixels to each side
lam_cent = 5303.
nside    = 300.

;;; Array to store slices of sky
big_sky = fltarr(2*nside+1,nfiles)
big_wav = fltarr(2*nside+1,nfiles)

;;; List of airmasses and median intensity values
big_airmass = fltarr(nfiles)
big_med     = fltarr(nfiles)

;;; Read in sky from each exposure.  Subtract off median sky value
;;; and shift in wavelength to match fiducial exposure.
for i=0,nfiles-1 do begin
   ; Read in sky
   temp_sky2d  = readfits(rootname(i)+'_sky.fits',hsky)
   temp_wav2d  = readfits(rootname(i)+'_wav.fits',hwav)
   temp_ccdnum = ccdnum(i)
   if (temp_ccdnum eq 1) then row = 200 else row = 900
   temp_skyslice = temp_sky2d(*,row)
   temp_wavslice = temp_wav2d(*,row)
   j = where(abs(temp_wavslice-lam_cent) eq min(abs(temp_wavslice-lam_cent)))
   j = j(0)
   temp_sky = temp_skyslice(j-nside:j+nside)
   temp_wav = temp_wavslice(j-nside:j+nside)
   ; Subtract off median and store
   temp_med = median(temp_sky)
   big_sky(*,i) = temp_sky / temp_med
   big_wav(*,i) = temp_wav
   ; Store airmass and median values
   big_airmass(i) = sxpar(hwav,'AIRMASS')
   big_med(i)     = temp_med
endfor

;;; Compute mean, median, and stddev at each pixel
med_sky  = median(big_sky,dim=2)
mean_sky = total(big_sky,2) / nfiles
sdev_sky = fltarr(2*nside+1)
for i=0,2*nside do sdev_sky(i) = stddev(big_sky(i,*))

;;; Mean wavelengths
mean_wav = total(big_wav,2) / nfiles

;;; Write files
writefits,outskyfile,mean_sky
writefits,outwavfile,mean_wav
stop
return
end
