Pro fors2_streakfixer,flatfile,slitgrid,fixedflatfile

;+--------------------------------------------------------------------------
;
; FORS2_STREAKFIXER    11/2006
;
; Interactively identify and remove streaks from a flat field image.
; Slit imperfections are assumed to be narrow features (such as due to
; dust on the slit), producing thin "grey" (constant slit position
; over all wavelengths) lines spanning the entire illuminated region.  The
; program is not intented to remove bad rows or other features associated
; with the CCD itself.
;
; INPUTS
;        flatfile      - String, FITS filename of flat from which streaks
;                        are to be removed.
;        slitgrid      - String, FITS filename of array of slit positions
;                        (see FORS2_SLITGRID).
; 
; OUTPUTS
;        fixedflatfile - String, name of FITS file to which to write 
;                        flat with streaks removed.
;
; HISTORY
;        Written 11/1/2006 GDB
;---------------------------------------------------------------------------

if (n_params() lt 3) then begin
   print,'CALLING SEQUENCE:'
   print,'     fors2_streakfixer,flatfile,slitgrid,fixedflatfile'
   return
end

; Read in flat and slitgrid
flat = readfors2(flatfile,hflat)
slit = readfors2(slitgrid,hslit)

sz  = size(flat)
xsz = sz(1)
ysz = sz(2)

; Display region
disp_xmin = 400
disp_xmax = 1650
disp_ymin = 15 ; 0
disp_ymax = 1008 ;1023

; Display scale
medvalue = median(flat(700:1400,500:800))
dispmin  = 0.9*medvalue
dispmax  = 1.1*medvalue

; Avoid edges when fitting streaks
xarr = lonarr(2048,1024)
for i=0,xsz-1 do xarr(i,*) = i
xfit_min = 100
xfit_max = xsz-100

; Select first defect

print,'Left click on a defect to remove.  Right click to exit'
chan,10
doit,flat(disp_xmin:disp_xmax,disp_ymin:disp_ymax),dispmin,dispmax
cursor,tmp_x,tmp_y,/device,/down
x = tmp_x + disp_xmin
y = tmp_y + disp_ymin

while(!mouse.button ne 4) do begin

   ; Extract a long section around the selected point
   slitselect = slit(x,y)
   region     = where(slit ge slitselect-7 and slit le slitselect+7 and $
                      xarr ge ((x-50) > xfit_min) and $
                      xarr le ((x+50) < xfit_max))
   regionslit = slit(region)
   regionflat = flat(region)

   ; Estimate depth of streak
   peak_ls    = where(slit ge slitselect-0.5 and slit le slitselect+0.5 and $
                      xarr ge ((x-50) > xfit_min) and $
                      xarr le ((x+50) < xfit_max))
   peak_value = median(flat(peak_ls))
   base_value = median(flat(region))
   depth      = peak_value - base_value

   ; Fit a Gaussian in slit position
   gfit = gaussfit(regionslit,regionflat,gterms,nterms=4,$
                   estimates=[depth,slitselect,1,base_value],yerror=gfiterr)
   tmp_ls = where(abs(regionflat-gfit) le 3*gfiterr)
   gfit = gaussfit(regionslit(tmp_ls),regionflat(tmp_ls),gterms,nterms=4,$
                   estimates=gterms,yerror=gfiterr)
   
   ; Evaluate fit
   goodfit = 1   ; Default is an acceptable fit
   gterms(2) = abs(gterms(2))
   if (gterms(2) lt 0. or $
       gterms(2) gt 10. or $
       (keyword_set(illum) and abs(gterms(0)) gt 0.1)) then goodfit = 0

   ; Divide through by Gaussian fit over entire flat
   if goodfit then begin
      fixls = where(slit ge gterms(1)-5*gterms(2) and $ 
                    slit le gterms(1)+5*gterms(2))
      for i=long(0),long(n_elements(fixls)-1) do begin
         slitval = slit(fixls(i))
         gval = 1 + gterms(0)*exp(-0.5*((slitval-gterms(1))/gterms(2))^2)
         flat(fixls(i)) = flat(fixls(i)) / gval
      endfor
      ; Print out parameters of defect removed
      print,gterms(0),gterms(1),gterms(2)
   endif else begin
      print,'Bad fit - no defect removed.'
   endelse

   ; Display flat with streak removed
   doit,flat(disp_xmin:disp_xmax,disp_ymin:disp_ymax),dispmin,dispmax

   ; Select next defect or exit
   print,'Left click on a defect to remove.  Right click to exit'
   cursor,tmp_x,tmp_y,/device,/down
   x = tmp_x + disp_xmin
   y = tmp_y + disp_ymin

endwhile

; Write output file
writefits,fixedflatfile,flat,hflat

return
end
