Pro fors2_wavegrid,exposure,slitgrid,wavegrid,ccd=ccd,slitrange=slitrange,$
                     list=list,outlist=outlist

;+-------------------------------------------------------------------------
;
; FORS2_WAVEGRID         4/2005
;
;
; Trace skylines or arclines in a 2D spectrum and create a 2D array of
; uncalibrated wavelengths.  The uncalibrated wavelength at a pixel is
; the x-intercept of the curve of constant wavelength that would pass
; through the center of that pixel.
;
; Must set either ccd=1 or ccd=2.
;
; The user may either select lines by hand or, optionally, identify
; lines in an input file (see below).
;
; INPUTS
;      exposure    - 2D array or FITS filename, raw object frame with
;                    pronounced skylines, or arc lamp exposure.
;      slitgrid    - 2D array or FITS filename, frame with slit
;                    position at each pixel.  Typically, the slit
;                    position is the x-intercept of a line of constant
;                    slit position that passes through the center of
;                    that pixel.
;      
; KEYWORDS
;      ccd         - MUST BE SET.  Scalar, CCD number (1 or 2).
;      slitrange   - 2-element array, minimum and maximum slit
;                    positions over which to trace sky or arc lines.
;                    Default is [10,1000] for CCD1 and [400,1000] for CCD2.
;      list        - String, name of 2-column ascii file containing x
;                    and y positions of lines to trace.  Default is
;                    to allow the user to select lines from a cut
;                    across the input exposure.
;      outlist     - String, name of 2-column ascii file to which to
;                    write the x and y positions of the identified
;                    lines.  If the keyword is set without any value,
;                    line positions will be written to 'list.dat'.
;
; OUTPUTS
;      wavegrid    - 2D array or FITS filename, uncalibrated wavelengths.
;                    If given as a string, the output will be written
;                    to a file with that name.
;
; HISTORY
;      Adopted from LONGSLIT_WAVEGRID 10/24/2006 GDB
;--------------------------------------------------------------------------

if (n_params() lt 3) then begin
 print,'CALLING SEQUENCE: '
 print,'   fors2_wavegrid,exposure,slitgrid,wavegrid,ccd=ccd,'
 print,'                  slitrange=slitrange,list=list,outlist=outlist'
 print,'   NOTE: Must set ccd keyword.'
 return
endif

;;; Is this for CCD1 or CCD2?
if keyword_set(ccd) then begin
   case ccd of
      1:    range = [10,1000]
      2:    range = [400,1000]
      else: begin
               print,'Unrecognized ccd number.  Must be either 1 or 2.'
               return
            end
   endcase
endif else begin
   print,'Must set either ccd=1 or ccd=2.'
   return
endelse

;;; Read in arrays, if necessary.

if (size(exposure,/tname) eq 'STRING') then begin
   checkexists = findfile(exposure,count=nfiles)
   if (nfiles eq 0) then begin
      print,'Exposure file not found.'
      return
   endif else begin
      obj = readfits(exposure,hobj)
   endelse
endif else begin
   obj = exposure
endelse

if (size(slitgrid,/tname) eq 'STRING') then begin
   checkexists = findfile(slitgrid,count=nfiles)
   if (nfiles eq 0) then begin
      print,'Slitgrid file not found.'
      return
   endif else begin
      slit = readfits(slitgrid,hslit)
   endelse
endif else slit = slitgrid

;;; Remove pre/overscan regions from sky/arc exposure, if not already
;;; done.
sz  = size(obj)
xsz = sz(1)
ysz = sz(2)
if (ysz ne 1024) then begin
   obj = obj(*,5:1028)
   sz  = size(obj)
   xsz = sz(1)
   ysz = sz(2)
endif

;;; Get screen size
device,get_screen_size=ssz

;;; Take a cut along the sky and identify lines, or read in line
;;; positions from a file.
if keyword_set(list) then begin
   templ =  {   $
          version: 1.0, $
          datastart: 0L, $
          delimiter: ' ', $
          missingvalue: -99.0, $
          commentsymbol: '#', $
          fieldcount: 2L, $
          fieldtypes: [4L,4L], $
          fieldnames: ['x','y'], $
          fieldlocations: [6L,23L], $
          fieldgroups: [0L,1L] $
        }
   position = read_ascii(list,template=templ)
   linex    = position.x
   liney    = position.y
   nlines   = n_elements(linex)
endif else begin
   ycut_cent = round(0.75*ysz)
   cut = median(obj(*,ycut_cent-2:ycut_cent+2),dim=2)
   z   = lindgen(xsz)
   window,10,xs=ssz(0),ys=(ssz(1) < 600)
   !p.multi = [0,1,2]
   plot,z,cut,/xstyle,title='Select lines to trace'
   plot,z,cut,/xstyle,yrange=[min(cut),min(cut)+1000]
   print,'Left-click on lines to trace.  Lines should be fairly well spaced.'
   print,'Right-click to exit.'
   getpoints,linez,linecounts,/plotx
   nlines = n_elements(linez)
   linex = linez
   liney = fltarr(nlines) + ycut_cent
endelse

;;; Optional - write line positions to a file.
if keyword_set(outlist) then begin
   if (size(outlist,/tname) eq 'STRING') then out_list = outlist $
      else out_list = 'list.dat'
   openw,list_lun,out_list,/get_lun
   for i=0,nlines-1 do printf,list_lun,linex(i),liney(i)
   close,list_lun
endif

;;; Optional - Manually set range of slit positions over which to
;;;            trace lines.
if keyword_set(slitrange) then range=slitrange

;;; Trace lines
chan,11
traceline,obj,linex,liney,xtr,ytr,slittr,/ydir,nside=10,$
   /bygauss,median=3,coord=slit,cbound=range,merit=merit
pixtr = ytr
ztr   = xtr

;;; Fit polynomials.  Reject outliers and iterate until 
;;; fit converges.
trace_order = 2
b           = fltarr(trace_order+1,nlines)
berr        = fltarr(trace_order+1,nlines)
zfit        = 0*ytr
nacross     = floor(sqrt(nlines))
ndown       = ceil((1.*nlines)/nacross)
chan,11
!p.multi    = [0,nacross,ndown]
for i=0,nlines-1 do begin
   temp_ls  = where(merit(*,i) ne 0,n_temp)
   temp_pix = pixtr(temp_ls,i)
   temp_z   = ztr(temp_ls,i)
   fit_converged = 0
   old_zfit      = 0
   n_iter        = 0
   good_ls       = indgen(n_temp)
   if (n_temp lt 20) then begin  ; Added 3/6/2012 GDB
      ; Flag bad fit
      print,'Bad fit - rejecting.'
      temp_b(*)     = -9999
      fit_converged = 1
   endif   
   while not(fit_converged) do begin
      temp_b = poly_fit(temp_pix(good_ls),temp_z(good_ls),trace_order,$
                        yerror=zerr,sig=temp_berr)
      temp_zfit   = 0*temp_pix
      for j=0,trace_order do temp_zfit = temp_zfit + temp_b(j)*temp_pix^j
      fit_diff = temp_zfit - old_zfit
      if (max(abs(fit_diff)) eq 0) then begin
         fit_converged = 1
      endif else begin
         good_ls = where(abs(temp_z-temp_zfit) le 3*zerr,n_good)
         ;if (n_good lt trace_order+1) then begin
         if (n_good lt 20) then begin  ; Changed 3/6/2012 GDB
            ; Flag bad fit
            print,'Bad fit - rejecting.'
            temp_b(*)     = -9999
            fit_converged = 1
         endif
      endelse
      n_iter = n_iter + 1
      if (n_iter ge 10) then fit_converged = 1
   endwhile
   b(*,i)    = temp_b
   berr(*,i) = temp_berr
   ; Plot residuals
   temp_zresid = temp_z - temp_zfit
   plot,temp_pix,temp_zresid,/xstyle,/ystyle,psym=3,$
      title='x = '+strtrim(linex(i),2)+', y = '+strtrim(liney(i),2),$
      charsize=2
endfor

;;; Fit coefficients as a function of intercept
; Reject bad fits
fitls = where(b(0,*) ne -9999 and b(0,*) ge -500 and b(0,*) le 1500)
b0 = b(0,fitls)
q  = fltarr(trace_order,4)
window,10,xs=(700 < ssz(0)),ys=(700 < ssz(1))
!p.multi = [0,1,1]
for i=0,trace_order-1 do begin
   print,' '
   print,'Fitting term '+strtrim(i+1,2)
   bi    = b(i+1,fitls)
   bierr = berr(i+1,fitls)
   qi    = inter_poly_fit(b0,bi,2,include=inc,measure_errors=bierr)
   q(i,0:n_elements(qi)-1) = qi
endfor

;;; Construct the wavelength grid.
print,'Computing wave grid...'
x = indgen(xsz)
y = indgen(ysz)
wavearr = float(xy2a(x,y,q))

;;; Optional - write to file.
if (size(wavegrid,/tname) eq 'STRING') then $
   writefits,wavegrid,wavearr else wavegrid = wavearr

return
end
