Pro fors2_xy,wavfile,slitfile,tracefile,xyfile,ccd=ccd

;+---------------------------------------------------------------------------
;
; FORS2_XY        11/2006
;
; For each pixel in a 2D spectrum, determine the x,y coordinates of a
; second 2D spectrum that are nearest in slit position and wavelength.
;
; Previously geared to work with FORS2 taken for the Lya fluorescence
; project.  The second spectrum is the combined 2D spectrum.
;
; Currently geared for lensed z~4 LBG J0332-3557.
;
; INPUTS
;        wavfile   - String, FITS filename of 2D wavelength solution
;        slitfile  - String, FITS filename of slitgrid for the input
;                    2D spectrum.
;        tracefile - String, name of ASCII file containing information
;                    about the object trace.  Information in this file
;                    is used to convert the input slit coordinates to
;                    'object-centric' slit coordinates.
;
; KEYWORDS
;        ccd       - Scalar, CCD numer (1 or 2).  If not set, CCD number
;                    will be read from the subfile header.
;
; OUTPUTS
;        xyfile    - String, FITS filename of 3D xy map.  The first
;                    two dimensions are the same size as the input 2D
;                    array.  (*,*,0) and (*,*,1) contain the x- and
;                    y-coordinates, respectively, of the pixels in the
;                    second (output) array that best match the input
;                    wavelength and object-centric slit coordinates.
;
; HISTORY
;        Written 11/12/2006 GDB
;----------------------------------------------------------------------------

if (n_params() lt 4) then begin
   print,'CALLING SEQUENCE:'
   print,'   fors2_xy,wavfile,slitfile,tracefile,xyfile,ccd=ccd'
   return
endif

;;; Read in input arrays

wav  = readfits(wavfile,hwav)
slit = readfits(slitfile,hslit)
sz   = size(slit)
xsz  = sz(1)
ysz  = sz(2)

;;; Determine which CCD

if keyword_set(ccd) then begin
   if not(ccd eq 1 or ccd eq 2) then begin
      print,'Keyword ccd must be 1 or 2'
      return
   endif
   ccdnum=ccd 
endif else begin
   extname     = strtrim(sxpar(hdr,'EXTNAME',count=n_extname),2)
   if (n_extname ne 0) then begin
      case extname of
         'CHIP1': ccdnum = 1
         'CHIP2': ccdnum = 2
         else:    begin
                     print,'Unknown CCD number.  Try setting ccd keyword.'
                     return
                  end
      endcase
   endif else begin
      print,'Unknown CCD number.  Try setting ccd keyword.'
      return
   endelse
endelse

;;; Read in trace information

trace_templ = {   $
          version: 1.0, $    
          datastart: 0L, $   
          delimiter: ' ', $
          missingvalue: -99.0, $
          commentsymbol: '#', $
          fieldcount: 1L, $
          fieldtypes: 4L, $
          fieldnames: 'value', $
          fieldlocations: 0L, $
          fieldgroups: [0L] $
        }
trace_data = read_ascii(tracefile,template=trace_templ)
; The first numbers are the polynomial fit to the object trace.  The
; last number is the object centroid, which should be nearly identical
; to the zeroeth term in the trace coefficients.
all_numbers  = trace_data.value
n_numbers    = n_elements(all_numbers)
trace_coeffs = all_numbers(0:n_numbers-2)
n_coeffs     = n_elements(trace_coeffs)

;;; Shift and rotate CCD2

if (ccdnum eq 2) then begin
   ; Adjust numbering to be contiguous with CCD, accounting 
   ; for the chip gap.  For a mean separation of 480 microns
   ; (16 2x binned 15 micron pixels), and a clockwise rotation
   ; of 0.083 degrees, pixel (0,1023) of CCD2 should have
   ; slit position -15.5.  Offset in wavelength will have negligible
   ; affect on the slit position.  (Note: Rotation is accounted
   ; for by the fact that separate slit position grids were
   ; used for CCDs 1 and 2, and these contain the relative
   ; rotation.)
   slit = slit - 1023. - 15.5  ; For longslit LAE
endif

;;; Create object-centric slit coordinates.  For longslit LAE, let the
;;; QSO be at slit position = 900.  For LBG, let bright star be at 150

x = findgen(xsz)
slit_correction = fltarr(xsz)
for i=1,n_coeffs-1 do $
   slit_correction = slit_correction + trace_coeffs(i)*(x^i)
obj_slit = fltarr(xsz,ysz)
for i=0,xsz-1 do obj_slit(i,*) = slit(i,*) - slit_correction(i)
obj_slit = obj_slit - trace_coeffs(0) + 150.  ;900. for longslit LAE

;;; Output y-positions will be the slit positions rounded to the
;;; nearest integer.

y_position = round(obj_slit)

;;; Wavelength solution for output spectra.  Wavelengths will be
;;; a function of x only.  The wavelength coefficients were chosen 
;;; from one of the fitted wavelength solutions that had 
;;; a third-order term in the middle of the range of all solutions.

;; For longslit LAE
;out_xsz  = 2080
;out_wmin = 4457.
;xout     = dindgen(out_xsz)
;wcoeff   = [out_wmin,0.587164,3.42638e-05,-4.32060e-09,-5.76243e-13]

;; For LBG - 1028z grism
;out_xsz  = 2080
;out_wmin = 7814.15
;xout     = dindgen(out_xsz)
;wcoeff   = [out_wmin,0.79854201,4.3413809e-05,-7.5303476e-09]

; For LBG - 1200R grism
out_xsz  = 2080
out_wmin = 5860.63
xout     = dindgen(out_xsz)
wcoeff   = [out_wmin,0.67468532,4.2867930e-05,-7.1113243e-09]

out_wav  = dblarr(out_xsz)
for i=0,n_elements(wcoeff)-1 do out_wav = out_wav + (xout^i)*wcoeff(i)

;;; Compute x-positions by interpolating input wavelengths onto
;;; output grid

x_position = fltarr(xsz,ysz)
for i=0,ysz-1 do x_position(*,i) = interpol(xout,out_wav,wav(*,i))
x_position = round(x_position)

;;; Write xy position file

xy_arr = intarr(xsz,ysz,2)
xy_arr(*,*,0) = x_position
xy_arr(*,*,1) = y_position
writefits,xyfile,xy_arr,hwav

return
end
