Pro fors2_combine2d,exposurelist,outfluxfile,outerrfile,badmap1=badmap1,$
                    badmap2=badmap2

;+-------------------------------------------------------------------------
;
; Non-optimized version.  Can be deleted if optimized version is working.
;
;
; FORS2_COMBINE2D     11/2006
;
; Combine 2D spectra.  Currently geared to combine FORS2 data taken
; for Lya fluorescence project (specific output file size).
;
; INPUTS
;       exposurelist  - String, name of single-column ASCII file
;                       containing root names of files to be combined.
;                       Program will look for the following files
;                       for each root name listed:
;                          <root>_sub.fits  - 2D sky-subtraced array
;                          <root>_var.fits  - 2D variance array
;                          <root>_xy.fits   - 3D coordinate array
;
; KEYWORDS
;       badmap1       - String, FITS filename of bad pixel map for
;                       CCD1.  Pixels with zero value in this array
;                       will be discarded.
;       badmap2       - String, FITS filename of bad pixel map for
;                       CCD2.  Pixels with zero value in this array
;                       will be discarded.       
;
; OUTPUTS
;       outfluxfile   - String, name of FITS file to which to write
;                       combined 2D array.
;       outerrfile    - String, name of FITS file to which to write
;                       2D array of formal 1-sigma errors.
;
; HISTORY
;       Written 11/15/2006 GDB
;       BADMAP1/2 keywords added 5/11/2007 GDB
;--------------------------------------------------------------------------

if (n_params() lt 3) then begin
   print,'CALLING SEQUENCE: '
   print,'     fors2_combine2d,exposurelist,outfluxfile,outerrfile'
   return
endif

;;; Read in list of files to be combined
templ = {   $
          version: 1.0, $    
          datastart: 0, $   
          delimiter: ' ', $
          missingvalue: '****', $
          commentsymbol: '#', $
          fieldcount: 1, $
          fieldtypes: [7], $
          fieldnames: ['root'], $
          fieldlocations: [0], $
          fieldgroups: [0] $
        }
data     = read_ascii(exposurelist,template=templ)
rootname = data.root
nfiles   = n_elements(rootname)

;;; Read in bad pixel maps (optional)
if (keyword_set(badmap1) or keyword_set(badmap2)) then begin
   do_masking = 1
   if keyword_set(badmap1) then bad_map1 = readfits(badmap1) $
      else badmap1 = lonarr(2048,1024)
   if keyword_set(badmap2) then bad_map2 = readfits(badmap2) $
      else badmap2 = lonarr(2048,1024)
endif else begin
   do_masking = 0
endelse

;;; Output arrays

out_xsz   = 2080.
out_ysz   = 1800.
comb_flux = fltarr(out_xsz,out_ysz)
comb_err  = fltarr(out_xsz,out_ysz)

;;; In order to conserve memory, proceed in slices.  For each slice,
;;; all input files will be read in and only those pixels that fall in
;;; the slice will be kept.

slice_sz = 100.
n_slice  = ceil(out_ysz / slice_sz)

for i=0,n_slice-1 do begin

   y_min     = i*slice_sz
   y_max     = ((i+1)*slice_sz) < (out_ysz-1)
   slice_ysz = y_max - y_min + 1

   ; Create large enough arrays to hold information from all input
   ; arrays, although not all input arrays will contribute to the
   ; current slice.
   temp_n    = out_xsz * slice_ysz * nfiles
   x_list    = fltarr(temp_n)
   y_list    = fltarr(temp_n)
   flux_list = fltarr(temp_n)
   var_list  = replicate(-1.,temp_n)   ; Default is no data
   mask_list = 1 + lonarr(temp_n)      ; Default is no masking
   count     = 0                       ; Index counter

   ; Read in data for this slice
   for j=0,nfiles-1 do begin
      temp_xy   = readfits(rootname(j)+'_xy.fits')
      contrib_ls   = where(temp_xy(*,*,1) ge y_min and $
                           temp_xy(*,*,1) le y_max,n_contrib)
      ; Only read in the remaining files for this exposure if the xy
      ; array indicates that this exposure contributes to the current
      ; slice.
      if (n_contrib ne 0) then begin
         temp_x    = temp_xy(*,*,0)
         temp_y    = temp_xy(*,*,1)
         temp_flux = readfits(rootname(j)+'_sub.fits',hflux)
         temp_var  = readfits(rootname(j)+'_var.fits',hvar)
         x_list(count:count+n_contrib-1)    = temp_x(contrib_ls)
         y_list(count:count+n_contrib-1)    = temp_y(contrib_ls)
         flux_list(count:count+n_contrib-1) = temp_flux(contrib_ls)
         var_list(count:count+n_contrib-1)  = temp_var(contrib_ls)
         ; Masking bad pixels?
         if do_masking then begin
            ; Determine which CCD
            chip = strtrim(sxpar(hflux,'EXTNAME'),2)
            case chip of
               'CHIP1': mask_list(count:count+n_contrib-1) = $
                                                  bad_map1(contrib_ls)
               'CHIP2': mask_list(count:count+n_contrib-1) = $
                                                  bad_map2(contrib_ls)
               else:    begin
                           print,'Cannot determine which CCD for file '+$
                                 rootname(j)
                           return
                        end
            endcase
         endif
         ; Increment pixel count
         count = count + n_contrib
      endif
   endfor

   ; Truncate lists to those pixels with good data.  Minimum
   ; value for flux was determined empirically from the data.
   good_ls   = where(var_list gt 0 and flux_list gt -75 and mask_list eq 1)
   x_list    = x_list(good_ls)
   y_list    = y_list(good_ls)
   flux_list = flux_list(good_ls)
   var_list  = var_list(good_ls)

   ; Number of sigma away from median for rejection
   nsig = 5.

   ; Go line-by-line, pixel-by-pixel and compute the combined flux
   for k=y_min,y_max do begin
      line_ls   = where(y_list eq k,n_line)
      if (n_line ne 0) then begin
         line_x    = x_list(line_ls)
         line_flux = flux_list(line_ls)
         line_var  = var_list(line_ls)
         for l=0,out_xsz-1 do begin
            pix_ls = where(line_x eq l,n_pix)
            if (n_pix ne 0) then begin
               pix_flux = line_flux(pix_ls)
               pix_var  = line_var(pix_ls)
               med      = median(pix_flux,/even)
               if (n_pix gt 2) then begin
                  if (n_pix ge 4) then sdev = robust_sigma(pix_flux) $
                     else sdev = stddev(pix_flux)
                  keep_ls = where(abs(pix_flux-med) lt nsig*sdev,n_keep)
               endif else begin
                  keep_ls = lindgen(n_pix)
                  n_keep  = n_pix
               endelse
               if (n_keep ne 0) then begin
                  keep_flux   = pix_flux(keep_ls)
                  keep_var    = pix_var(keep_ls)
                  keep_invvar = 1./keep_var
                  comb_flux(l,k) = total(keep_flux*keep_invvar)/$
                                   total(keep_invvar)
                  comb_err(l,k)  = sqrt(1./total(keep_invvar))
               endif else begin
                  comb_flux(l,k) = 0.
                  comb_err(l,k)  = -1.
               endelse
            endif
         endfor
      endif
   endfor   

endfor

; Write output files

writefits,outfluxfile,comb_flux
writefits,outerrfile,comb_err


return
end
